<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/*
	Class: OT_Controller
	
	Provides a base class for all controllers that check user login and valid domain.
*/

class OT_Controller extends CI_Controller {

	private $ignore_pages = array('user/login');
	
	function __construct()
	{
		parent::__construct();
//		$this->output->enable_profiler(TRUE);

		if (!file_exists(FCPATH . APPPATH . 'config/client.db.php') || !file_exists(FCPATH . APPPATH . 'config/client.php'))
		{
			file_put_contents(FCPATH . 'installer.php', 'Outputtime Installer');
			chmod(FCPATH . 'installer.php', 0777);
			redirect('/');
			return false;
		}

		$this->load->library(OTBASE_PATH.'request');
		$this->load->response();
		// validate token if request via ajax
		if ($this->request->get_type()=='AJAX' && $this->input->is_valid_token()==false)
		{
			$this->response->error('INVALID_REQUEST', true);
			return false;
		}
		// Validate domain
		$this->load->library(OTBASE_PATH.'OT_App');
		if (!$this->ot_app->valid())
		{
			return false;
		}
		
		$this->load->model('user');
		//Autologin users if cookie exists

		if ( ! $this->session->userdata("user"))
			$this->user->login_with_cookie();

		if (!$this->do_auth()) {
			return false;
		}
		$this->load->library('uri');
		$controller = $this->uri->segment(1);
		$view = $this->uri->segment(2);

		if ($this->session->userdata('lic_invalid') && $controller!='report' && !in_array($view, array('progress','logout'))) {
			redirect('inactive');
		}

		$this->load->library(OTBASE_PATH.'OT_User');
		if ($this->ot_user->get('user_id')) {
			$this->load->model('timesheets');
		}
	}
	
	// check valid ajax request
	public function _remap($method, $params = array())
	{
		if ($this->is_valid_access($method, $params)) {
			return call_user_func_array(array($this, $method), $params);
		}
	}
	// check valid access
	public function is_valid_access($method, $params = array())
	{
		$methods = array('index', 'template', 'logout', 'login','approve');
		if (method_exists($this, $method))
		{
			if (!in_array($method, $methods))
			{
				if ($this->request->get_type()!='AJAX' && $this->input->is_valid_token()==false)
				{
					$this->response->error('PERMISSION_DENIED', true);
					return false;
				}
			}
			
			return true;
		}
		
		$this->response->error('PAGE_NOT_FOUND', 404);
	}

	public function set_ignore_pages($pages = array())
	{
		if ($pages)
		{
			if (!is_array($pages))
			{
				$this->ignore_pages[] = $pages;
			}
			else
			{
				$this->ignore_pages = array_merge($this->ignore_pages, $pages);
			}
		}
	}

	public function do_auth()
	{
		$this->load->library(OTBASE_PATH.'OT_User');
		if ( ! $this->ot_user->get('user_id'))
		{
			if (!in_array($this->uri->uri_string(), $this->ignore_pages)) {

				$this->load->helper('file');

				if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
					$ip = $_SERVER['HTTP_CLIENT_IP'];
				} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
					$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
				} else {
					$ip = $_SERVER['REMOTE_ADDR'];
				}
				/*write_file('uploads/session_expiry.txt', date('Y-m-d H:i:s').PHP_EOL.$ip.PHP_EOL.'REQUEST URI:'.$_SERVER['REQUEST_URI'].PHP_EOL.'URI:'.$this->uri->uri_string().PHP_EOL.lang('SESSION_EXPIRED').PHP_EOL, "a+");
*/
				$this->response->info('SESSION_EXPIRED');
				$ind_pos = strpos($_SERVER['REQUEST_URI'],'index.php');
				if ($this->request->get_type()!='AJAX' && strpos($_SERVER['REQUEST_URI'],'attachment/')===false && strpos($_SERVER['REQUEST_URI'],'timesheet/progress')===false && strlen(substr($_SERVER['REQUEST_URI'],$ind_pos+9))>2) {
					$red_url = substr($_SERVER['REQUEST_URI'],(strpos($_SERVER['REQUEST_URI'],'index.php')-1));	
					$this->session->set_flashdata('redirect_url', $red_url);
				}

				$this->response->write(array('redirect'=>site_url('login')));
				return false;
			}
			$this->session->keep_flashdata('redirect_url');
		}

		return true;
	}
}

/*  Administrator_role
 * 
 * This controller restrict user those not have administrator role.
 */
class Administrator_role extends OT_Controller {
	function __construct()
	{
		parent::__construct();
		$this->load->response();
		
		// check valid user role
		if (!is_admin())
		{
			$this->response->error("PERMISSION_DENIED");
			$this->response->write(array('redirect' => site_url()));
			return false;
		}
	}
}

/*  Member_role
 * 
 * This controller restrict user those not have member role.
 */
class Member_role extends OT_Controller {
	function __construct()
	{
		parent::__construct();
		$this->load->response();
		
		// check valid user role
		if (!is_member())
		{
			$this->response->error("PERMISSION_DENIED");
			$this->response->write(array('redirect' => site_url()));
			return false;
		}
	}
}

/*  Pm_role
 * 
 * This controller restrict user those not have project manager role. 
 */
class Pm_role extends OT_Controller {
	function __construct()
	{
		parent::__construct();
		$this->load->response();
		
		// check valid user role
		if (!is_pm())
		{
			$this->response->error("PERMISSION_DENIED");
			$this->response->write(array('redirect' => site_url()));
			return false;
		}
	}
}

/*  Pmf_role
 * 
 * This controller restrict user those not have project manager finance role.
 */
class Pmf_role extends OT_Controller {
	function __construct()
	{
		parent::__construct();
		$this->load->response();
		
		// check valid user role
		if (!is_admin() && !is_pmf())
		{
			$this->response->error("PERMISSION_DENIED");
			$this->response->write(array('redirect' => site_url()));
			return false;
		}
	}
}

/*  Secured_role
 * 
 * This controller restrict user those have member role.
 */
class Secured_role extends Ot_Controller {
	function __construct()
	{
		parent::__construct();
		
		// 	check valid user role
		if (is_member())
		{
			$this->response->error("PERMISSION_DENIED");
			$this->response->write(array('redirect' => site_url()));
			return false;
		}
	}
}
