<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

if (file_exists(APPPATH . '/libraries/saas_auth.php'))
	require_once(APPPATH . '/libraries/saas_auth.php');

class Auth {

	function __construct()
	{
		$this->CI =& get_instance();
		
		//load libraries
		$this->CI->load->helper('string');
		$this->CI->load->library(OTBASE_PATH.'OT_App');
        $this->CI->load->library(OTBASE_PATH.'OT_User');
	}
	
	public function encode($password)
	{
		  $this->CI->load->library('encrypt');
          return $this->CI->encrypt->sha1($password);
	}
	
	public function generate_auth_token($user_id, $email)
	{
		$auth_token = $email.strtotime("now").random_string('alnum', 5);
		
		return $this->encode($auth_token).'=='.$user_id;
	}
	
	public static function get_random_password($length = 8)
	{
		$salt = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
		$base = strlen($salt);
		$makepass = '';
		
		$random = random_string('alnum', $length+1);
		$shift = ord($random[0]);
		for ($i = 1; $i <= $length; ++$i)
		{
			$makepass .= $salt[($shift + ord($random[$i])) % $base];
			$shift += ord($random[$i]);
		}

		return $makepass;
	}
	
	function encrypt_decrypt($action, $string='') {
	    $output = false;
		$CI	=& get_instance();
		
		if( $action == 'encrypt' ) {
			$output = $this->get_random_password(5).strrev($string).$this->get_random_password(2);
			$output = str_replace('/','%s$',base64_encode($output));
		} else {
			$output = base64_decode(str_replace('%s$','/',$string));
			$output = strrev(substr($output,5,-2));		
		}
	
	    return $output;
	}
	
	public function get_timezone_date($date_format='', $user_zone = false)
	{
		$CI	=& get_instance();
		$CI->load->helper('date');
		
		$date_format= ($date_format)?$date_format:'Y-m-d H:i:s';
		if ($user_zone)
		{
			$user_zone = is_string($user_zone)?$user_zone:$CI->ot_user->get('time_zone_code');
		}
		
		if($user_zone)
		{
			$timestamp = strtotime(gmdate('Y-m-d H:i:s'));
			$daylight_saving = date('I'); 		
			$local_timestamp = gmt_to_local($timestamp, $user_zone, $daylight_saving);
			return date($date_format,$local_timestamp);
		}
		
		return gmdate($date_format);
	}

	public function parse_license($license="")
	{
		if ($eData=$this->get_license($license, true))
		{
			$eData	= explode("#", $eData);
			if (count($eData) == 5)
			{
				return array_combine(array('entity_id', 'expiry_date', 'max_users', 'plan', 'company_name'), $eData);
			}
		}

		return false;
	}

	public function get_license($data='', $isDecrypt=false)
	{
		if (function_exists ('get_license')) {
			return get_license($data, $isDecrypt);
		}
		
		if ($isDecrypt) {
			return $this->lic_decrypt($data);
		}
		return '';
	}

	public function lic_decrypt($data , $keyLen=12) {
		$res	= '';
		$key	= substr($data, 0, $keyLen);
		$data	= base64_decode(substr($data, $keyLen));

		for($i=0; $i<strlen($data); $i++)
		{
			$c	= ord(substr($data, $i));
			$c	-= ord(substr($key, (($i + 1) % $keyLen)));
			$res	.= chr(abs($c) & 0xFF);
		}

		return $res;
	}

	public function license_data() {
		$entity = $this->CI->session->userdata('entity');
		$this->CI->session->set_userdata('lic_invalid',1);
		
		if (empty($entity->entity_id)) return false;
		
		$this->CI->db->select('max_users, DATE_FORMAT(exp_date, "%Y-%m-%d") AS exp_date, plan, license_key, company_name', false);
		$this->CI->db->from('entity_accounts');
		$this->CI->db->where(array('entity_id' => $entity->entity_id, 'is_active' => 1));
		$row = $this->CI->db->get()->row_array();

		if (!$row) return false;

		if (!defined('APP_EXP') || !empty($row['license_key'])) {
			if (!$row['license_key']) return false;
			$lic=$this->parse_license($row['license_key']);
			
			// check lic matched entity
			if ($lic['entity_id']!=$entity->entity_id) return false;
			$row = $lic;
			$row['exp_date'] 	= $lic['expiry_date'];
			// onpremises pro version
			if (!defined('APP_EXP') && $lic['plan']=='P')
				$lic_invalid = 0;
		}
		if (!isset($lic_invalid)) {
			$lic_invalid 	= (strtotime($row['exp_date']) >= strtotime($this->CI->auth->get_timezone_date('Y-m-d')))?0:1;
			// $lic_invalid 	= (strtotime($row['exp_date']) >= strtotime(date("Y-m-d")))?0:1;
		}
		$entity->exp_date 		= $row['exp_date'];
		$entity->max_users 		= $row['max_users'];
		$entity->plan 			= $row['plan'];
		$entity->company_name	= $row['company_name'];
		
		$this->CI->session->set_userdata('entity', $entity);
		$this->CI->session->set_userdata('lic_invalid',$lic_invalid);
		return (!$lic_invalid)?$entity:$lic_invalid;
	}
}
