<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class project_table extends Base_table {

	function __construct()
	{
		parent::__construct(array('tbl'=>'project', 'tbl_key'=>'project_id'));
		$this->CI->load->library('email');
		$config = array(
		    'mailtype'    => 'html',
		    'charset'    => 'utf-8'
		);
		$this->CI->email->initialize($config);
    }

 	public function formatdata(&$data) 
	{
		if (empty($data[$this->tbl_key]))
		{
			// New project default datas
			$data['members_count']		= 1;
 			$data['is_active'] 			= true;		

 			
			if (!empty($data['client_id']))
			{
				$this->CI->db->where('client_id', $data['client_id']);
			}
			$this->CI->db->or_where('entity_id', $this->entity_id);
			$this->CI->db->select('budget_type, invoice_type');
			$this->CI->db->from('project');
			$this->CI->db->order_by('created_date','desc');
			$this->CI->db->limit(1);
			$query = $this->CI->db->get();
			
			$result = $query->row_array();
			if (!$result)
			{
				$result['budget_type'] = 'AT';
				$result['invoice_type'] = 'T';
			}

 			$data['budget_type']  = !empty($data['budget_type'])?$data['budget_type']:$result['budget_type'];
 			$data['invoice_type'] = !empty($data['invoice_type'])?$data['invoice_type']:$result['invoice_type'];
		}
		
		if (!empty($data['project_code'])) {
			$data['project_code'] = substr($data['project_code'],0,6).' - ';
		}

		$data['is_active'] = ($data['is_active'])?1:0;

	}

    function save(&$data) {
		$project_id = !empty($data[$this->tbl_key])?$data[$this->tbl_key]:0;
	
		if (!license_limit($project_id, 'projects')) {
			$this->CI->response->error(sprintf(lang('UPGRADE_YOUR_VERSION'),'projects'));
			return false;
		}

		if ($project_id = parent::save($data))
		{
			if (empty($data[$this->tbl_key]))
			{
				/*$this->CI->load->library('project_task_types_table');
				
				
				if (! $this->CI->project_task_types_table->store_default_data($project_id))
				{
						return false;
				}*/
				
				// Insert project activity
				$act_data = array(
					'project_id' 	=> $project_id,
					'a_date'		=> $this->cur_time,
					'a_mode'		=> empty($data[$this->tbl_key])?'PN':'PP'
				);
				
				if (! $this->insert_project_activity($act_data))
				{
						return false;
				}
				activity_log('PN',$project_id, $this->CI->ot_user->get('first_name').' '.$this->CI->ot_user->get('last_name').' created project '.$data['project_name']);
			} else {
				activity_log('PP',$project_id, $this->CI->ot_user->get('first_name').' '.$this->CI->ot_user->get('last_name').' edited project properties of '.$data['project_name']);
			}

			return $project_id;
        }
		$this->response->error('Save error');
        return false;
    }
    
	public function update_budget_value($data, $update_actual=true)
	{
		$this->CI->db->select('budget_type');
		$this->CI->db->from('project');
		$this->CI->db->where(array('project_id' => $data['project_id'], 'entity_id' => $this->entity_id));
		$row = $this->CI->db->get()->row();
		$select = '';
		$budeget_type = $row->budget_type;
		if (!in_array($budeget_type, array('HA', 'AA')))
		{
			switch ($budeget_type)
			{
				case 'HT':
						$select = 'sum(estimated_hours) as budget_value';
				case 'AT':
						$select = ($select)?$select:'sum(estimated_hours*hourly_rate) as budget_value';
						$this->CI->db->select($select, false);
						$this->CI->db->from('project_task_types');
						$this->CI->db->where(array('project_id' => $data['project_id']));
						$row = $this->CI->db->get()->row();
						$budget_value = $row->budget_value;
					break;
				case 'HP':
					$select = 'sum(estimated_hours) as budget_value';
				case 'AP':
						$select = ($select)?$select:'sum(estimated_hours*hourly_rate) as budget_value';
						$this->CI->db->select($select, false);
						$this->CI->db->from('project_users');
						$this->CI->db->where(array('project_id' => $data['project_id']));
						$row = $this->CI->db->get()->row();
						$budget_value = $row->budget_value;
					break;
			}
			
			$this->CI->db->set('budget_value', $budget_value);
			$this->CI->db->where(array('project_id' => $data['project_id'], 'entity_id' => $this->entity_id));
			$this->CI->db->update('project');
		}
		
		if ($update_actual)
			$this->update_actual_value($data, $budeget_type);
							
		return true;
	}
	
	
	
	public function update_actual_value($data, $budeget_type ='')
	{

		if(!$budeget_type) {
			$this->CI->db->select('budget_type');
			$this->CI->db->from('project');
			$this->CI->db->where(array('project_id' => $data['project_id'], 'entity_id' => $this->entity_id));
			$row = $this->CI->db->get()->row();
			$budeget_type = $row->budget_type;
		}

		$select = '';
		switch ($budeget_type)
		{
			case 'HA':
			case 'HT':
					$select = 'sum(t.hours) as actual_value';
			case 'AA':			
			case 'AT':
					$select = ($select)?$select:'sum(ty.hourly_rate * t.hours) as actual_value';
					$this->CI->db->select($select, false);
					$this->CI->db->from('project_task_types AS ty');
					$this->CI->db->join('time_sheet As t', 'ty.entity_id=t.entity_id and ty.project_id=t.project_id AND ty.task_type_id = t.task_type_id', 'left');
					$this->CI->db->where(array('ty.entity_id' => $this->entity_id, 'ty.project_id' => $data['project_id']));
					$row = $this->CI->db->get()->row();
					$actual_value = $row->actual_value;
				break;
			case 'HP':
					$select = 'sum(t.hours) as actual_value';
			case 'AP':
					$select = ($select)?$select:'sum(tu.hourly_rate * t.hours) as actual_value';
					$this->CI->db->select($select, false);
					$this->CI->db->from('project_users AS tu');
					$this->CI->db->join('time_sheet As t', 'tu.entity_id=t.entity_id and tu.project_id = t.project_id AND t.user_id = tu.user_id', 'left');
					$this->CI->db->where(array('tu.entity_id' => $this->entity_id, 'tu.project_id' => $data['project_id']));
					$row = $this->CI->db->get()->row();
					$actual_value = $row->actual_value;
				break;
		}

		$this->CI->db->set('actual_value', $actual_value);
		$this->CI->db->where(array('project_id' => $data['project_id'], 'entity_id' => $this->entity_id));
		$this->CI->db->update('project');
				
		return true;
	}
    
    public function insert_project_activity($act_data)
    {
    	
    	$data = array(
				'entity_id' 	=> $this->entity_id,
				'user_id'		=> $this->user_id,
				'user_ip'		=> $_SERVER['REMOTE_ADDR']
			);
		$data = array_merge($data, $act_data);
		
		if (!$this->CI->db->insert('project_activity', $data))
		{
			return false;
		}
		
		return true;
    }
 
	function delete($project_id, $params= array())
	{
		$pr_where = $where = array('project_id'=> $project_id,'entity_id'=>$this->entity_id);
		// check valid delete
		if ($params) $pr_where = array_merge($pr_where,$params);
		$this->CI->db->select('project_name');
		$this->CI->db->where($pr_where);
		$query= $this->CI->db->get('project');
		$data= $query->row_array();

		if (!$data) return false;
		
		// delete project related data
		$this->CI->load->library('project_task_types_table');
		if (!$this->CI->project_task_types_table->delete($project_id))
		{
			return false;
		}
		$this->CI->load->library('project_users_table');
		if (!$this->CI->project_users_table->delete($project_id))
		{
			return false;
		}
		$this->CI->db->delete('project_activity',array('entity_id'=>$this->entity_id, 'project_id'=> $project_id));
	
		$this->CI->db->delete('attachment', array('entity_id'=>$this->entity_id, 'module_type' => $project_id,'module_id' => 'P'));
		$this->CI->db->delete('comments', array('entity_id'=>$this->entity_id, 'module_type' => $project_id,'module_id' => 'P'));
		$this->CI->db->delete('project_expense', $where);		
		
		//timesheet detail delete
		$this->CI->db->delete('time_sheet', $where);
		$this->CI->db->query('DELETE tsd.* FROM '.$this->CI->db->dbprefix('time_sheet_detail').' AS tsd left join '.$this->CI->db->dbprefix('time_sheet').' AS t ON tsd.time_sheet_id = t.time_sheet_id where t.time_sheet_id is NULL');
		$this->CI->db->delete('project_task', $where);
		$this->CI->db->query('DELETE ptu.* FROM '.$this->CI->db->dbprefix('project_task_users').' AS ptu left join '.$this->CI->db->dbprefix('project_task').' AS pt ON ptu.task_id = pt.task_id where pt.task_id is NULL');
		
		$this->CI->db->delete('project', $where);
		activity_log('PD',$project_id, $this->CI->ot_user->get('first_name').' '.$this->CI->ot_user->get('last_name').' deleted project '.$data['project_name']);
		return true;
	}
}
